/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 931 $ $LastChangedDate: 2002-09-11 10:32:30 +0200 (Wed, 11 Sep 2002) $
 *
 * Standard library for Gont, Array module C implementation.
 */

#include <gont/runtime.h>
#include <gont/array.h>

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

typedef struct gont_array *t;

t Array___alloc(gont_int sz)
{
	t ret;

	ret = gont_xmalloc(sizeof(*ret));
	
	if (sz == 0)
		ret->data = NULL;
	else
		ret->data = gont_xmalloc(sizeof(gont_int) * sz);

	ret->size = ret->length = sz;

	return ret;
}

t Array__make(gont_int sz, gont_int ini)
{
	t ret = Array___alloc(sz);
	gont_int i;

	for (i = 0; i < sz; i++)
		ret->data[i] = ini;

	return ret;
}

gont_int Array__length(t ar)
{
	return ar->length;
}

void Array__set(t ar, gont_int idx, gont_int val)
{
	if (idx >= ar->length)
		gont_invalid_argument("Array::set");
		
	ar->data[idx] = val;
}

gont_int Array__get(t ar, gont_int idx)
{
	if (idx >= ar->length)
		gont_invalid_argument("Array::get");
		
	return ar->data[idx];
}

static void resize_exctly_to(t ar, gont_int newsz)
{
	gont_int *newdata = gont_xmalloc(newsz * sizeof(gont_int));
	gont_int len = ar->length;

	if (newsz < ar->size)
		len = newsz;
		
	memcpy(newdata, ar->data, len * sizeof(gont_int));

	ar->length = ar->size = newsz;
	ar->data = newdata;
}

void Array__really_resize(t ar, gont_int newsz, gont_int ini)
{
	gont_int i = ar->length;

	resize_exctly_to(ar, newsz);

	while (i < newsz)
		ar->data[i++] = ini;
}

void Array___resize(t ar, gont_int newsz)
{
	if (newsz > ar->size) {
		if (newsz < 5)
			resize_exctly_to(ar, 10);
		else
			resize_exctly_to(ar, newsz * 2);
	}

	ar->length = newsz;
}

void Array__resize(t ar, gont_int newsz, gont_int ini)
{
	gont_int i = ar->length;

	Array___resize(ar, newsz);

	while (i < newsz)
		ar->data[i++] = ini;
}

void Array__append(t ar, gont_int ini)
{
	if (ar->length == ar->size)
		Array__resize(ar, ar->length + 1, ini);
	else
		ar->data[ar->length++] = ini;
}

void Array__blit(t src, gont_int srcoff, t dst, gont_int dstoff, 
		 gont_int len)
{
	if (srcoff > src->length || dstoff > dst->length ||
	    srcoff + len > src->length || dstoff + len > dst->length)
		gont_invalid_argument("Array::blit");
	memmove(dst->data + dstoff, src->data + srcoff, 
		len * sizeof(gont_int));
}

void Array__fill(t a, gont_int off, gont_int len, gont_int v)
{
	
	if (off > a->length || off + len > a->length)
		gont_invalid_argument("Array::fill");

	if (len) {
		gont_int *ptr = a->data + off;
		while (len--)
			*ptr++ = v;
	}
}
