/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 1048 $ $LastChangedDate: 2002-10-26 23:19:49 +0200 (Sat, 26 Oct 2002) $
 *
 * Runtime environment for gont.
 */

#include <gont/runtime.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

struct exit_chain {
	struct exit_chain *next;
	void (*fnc)(void*);
	void *data;
};

static struct exit_chain *exit_chain;

int gont_argc;
char **gont_argv;
char **gont_environ;

#define FALLBACK_CHARSET "ISO-8859-1"

void gont_init_runtime(void)
{
	const char *charset;
	
	gont_init_memory_allocator();

	charset = gont_get_locale_charset();
	
	if (charset == NULL) {
#if 0
		fprintf(stderr, "Gont runtime: cannot determine charset from "
			"locale, falling back to `%s'\n", FALLBACK_CHARSET);
#endif
		charset = FALLBACK_CHARSET;
	}
		
	gont_default_charset = gont_make_charset(charset, 0, 0);

	if (gont_default_charset == NULL) {
		if (strcmp(charset, FALLBACK_CHARSET) != 0) {
			fprintf(stderr, "Gont runtime: iconv doesn't support "
				"charset `%s', falling back to `%s'\n",
				charset, FALLBACK_CHARSET);
			
			gont_default_charset = 
				gont_make_charset(FALLBACK_CHARSET, 0, 0);
			charset = FALLBACK_CHARSET;
		}

		if (gont_default_charset == NULL) {
			fprintf(stderr, "Gont runtime: iconv can't get `%s'\n",
				charset);
			gont_oops("broken iconv");
		}
	}

	gont_default_charset_with_approx = gont_make_charset(charset, 1, 0);
	if (gont_default_charset_with_approx == NULL)
		abort();
}

void gont_exit(void)
{
	exit(0);
}

void gont_set_argv(int argc, char **argv)
{
	gont_argc = argc;
	gont_argv = argv;
}

void gont_set_environ(char **environ)
{
	gont_environ = environ;
}

void gont_oops(const char *msg, ...)
{
	va_list ap;

	fputs("Gont runtime fatal error: ", stderr);
	va_start(ap, msg);
	vfprintf(stderr, msg, ap);
	va_end(ap);
	fputs("\n", stderr);

	abort();
}

void gont_run_exit_routines(void)
{
	struct exit_chain *p;
	
	while (exit_chain) {
		p = exit_chain;
		exit_chain = exit_chain->next;
		p->fnc(p->data);
	}
}

void gont_register_exit_routine(void (*fnc)(void*), void *data)
{
	struct exit_chain *c;

	c = gont_xmalloc(sizeof(*c));
	c->next = exit_chain;
	c->fnc = fnc;
	c->data = data;
	exit_chain = c;
}
