/* 
 * Copyright (c) 2002 Michal Moskal <malekith@pld-linux.org>
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Rev: 1033 $ $LastChangedDate: 2002-09-22 09:06:47 +0200 (Sun, 22 Sep 2002) $
 *
 * Standard library for Gont, Int64 module interface.
 */

/** Int64 provides support for operations on 64 bit integers, no matter
    how big native pointer is.  */

/** Type of 64 bit integers.  */
type t;

/** Return [a + b].  */
t add(t a, t b) = "Int64__add";

/** Return [a - b].  */
t sub(t a, t b) = "Int64__sub";

/** Return [a * b].  */
t mul(t a, t b) = "Int64__mul";

/** Return [a / b].  Raise [Invalid_argument] if [b == 0].  */
t div(t a, t b) = "Int64__div";

/** Return [a % b] (modulus).  Raise [Invalid_argument] if [b == 0].  */
t mod(t a, t b) = "Int64__mod";

/** Return -1, 0 or 1 if [a < b], [a == b] or [a > b] respectively.  */
int cmp(t a, t b) = "Int64__cmp";

/** Convert [s], from string decimal representation to 64 bit int.  Raise
    [Invalid_argument] if [s] is not valid integer literal.  If number starts
    with [0x], [0o] or [0b] radix of 16, 8 or 2, respectively, is assumed. */
t of_string(string s) = "Int64__of_string";

/** Extend native integer [i] to 64 bit.  */
t of_int(int i);

/** Convert [a] to string using radix [base].  */
string to_string(t a, int base) = "Int64__to_string";

/** Return [a + 1].  */
t succ(t a);

/** Return [a - 1].  */
t pred(t a);

/** Value of 0.  */
t zero;

/** Value of 1.  */
t one;

/** Value of -1.  */
t minus_one;
